@@ -1,10 +1,10 @@
 import os, time
-from libarchive import is_archive, Entry, SeekableArchive
+from libarchive import is_archive, Entry, SeekableArchive, _libarchive
 from zipfile import ZIP_STORED, ZIP_DEFLATED
 
 
 def is_zipfile(filename):
-    return is_archive(filename, formats=('zip', ))
+    return is_archive(filename, formats=('zip',))
 
 
 class ZipEntry(Entry):
@@ -60,41 +60,55 @@ class ZipEntry(Entry):
     CRC = property(_get_missing, _set_missing)
     compress_size = property(_get_missing, _set_missing)
 
-
+# encryption is one of (traditional = zipcrypt, aes128, aes256)
 class ZipFile(SeekableArchive):
-    def __init__(self, f, mode='r', compression=ZIP_DEFLATED, allowZip64=False):
-        super(ZipFile, self).__init__(f, mode=mode, format='zip', entry_class=ZipEntry, encoding='CP437')
-        if mode == 'w' and compression == ZIP_STORED:
-            # Disable compression for writing.
-            _libarchive.archive_write_set_format_option(self.archive._a, "zip", "compression", "store")
+    def __init__(self, f, mode='r', compression=ZIP_DEFLATED, allowZip64=False, password=None,
+        encryption=None):
         self.compression = compression
+        self.encryption = encryption
+        super(ZipFile, self).__init__(
+            f, mode=mode, format='zip', entry_class=ZipEntry, encoding='CP437', password=password
+        )
+        
 
-    getinfo     = SeekableArchive.getentry
+    getinfo = SeekableArchive.getentry
+
+    def set_initial_options(self):
+        if self.mode == 'w' and self.compression == ZIP_STORED:
+            # Disable compression for writing.
+            _libarchive.archive_write_set_format_option(self._a, "zip", "compression", "store")
+        
+        if self.mode == 'w' and self.password:
+            if not self.encryption:
+                self.encryption = "traditional"
+            _libarchive.archive_write_set_format_option(self._a, "zip", "encryption", self.encryption)
+          
+       
 
     def namelist(self):
-        return list(self.iterpaths)
+        return list(self.iterpaths())
 
     def infolist(self):
         return list(self)
 
     def open(self, name, mode, pwd=None):
-        if pwd:
-            raise NotImplemented('Encryption not supported.')
         if mode == 'r':
+            if pwd:
+                self.add_passphrase(pwd)
             return self.readstream(name)
         else:
             return self.writestream(name)
 
     def extract(self, name, path=None, pwd=None):
         if pwd:
-            raise NotImplemented('Encryption not supported.')
+            self.add_passphrase(pwd)
         if not path:
             path = os.getcwd()
         return self.readpath(name, os.path.join(path, name))
 
     def extractall(self, path, names=None, pwd=None):
         if pwd:
-            raise NotImplemented('Encryption not supported.')
+            self.add_passphrase(pwd)
         if not names:
             names = self.namelist()
         if names:
@@ -103,16 +117,16 @@ class ZipFile(SeekableArchive):
 
     def read(self, name, pwd=None):
         if pwd:
-            raise NotImplemented('Encryption not supported.')
-        return self.read(name)
+            self.add_passphrase(pwd)
+        return super(ZipFile, self).read(name)
 
     def writestr(self, member, data, compress_type=None):
-        if compress_type != self.compression:
+        if compress_type != self.compression and not (compress_type is None):
             raise Exception('Cannot change compression type for individual entries.')
         return self.write(member, data)
 
     def setpassword(self, pwd):
-        raise NotImplemented('Encryption not supported.')
+        return self.set_passphrase(pwd)
 
     def testzip(self):
         raise NotImplemented()
